"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AccountKeyFactory = void 0;
exports.parseAccountKey = parseAccountKey;
const index_js_1 = require("../field/index.js");
const index_js_2 = require("../util/index.js");
const accountkey_js_1 = require("./accountkey.js");
class AccountKeyFactory extends index_js_1.FieldSetFactory {
    constructor() {
        const requiredFields = ["type"];
        super(requiredFields);
        this.add(accountkey_js_1.AccountKeyNil);
        this.add(accountkey_js_1.AccountKeyLegacy);
        this.add(accountkey_js_1.AccountKeyPublic);
        this.add(accountkey_js_1.AccountKeyFail);
        this.add(accountkey_js_1.AccountKeyWeightedMultiSig);
        this.add(accountkey_js_1.AccountKeyRoleBased);
    }
    _fromObject(fields) {
        // In AccountKeyWeightedMultiSig, alias weightedPublicKeys -> keys for compatibility.
        // 'weightedPublicKeys' is used in caver and klaytn node.
        // 'keys' is used in this SDK.
        if (fields.type == index_js_2.AccountKeyType.WeightedMultiSig && fields.weightedPublicKeys) {
            fields.keys = fields.weightedPublicKeys;
        }
        return super.fromObject(fields);
    }
    static fromObject(fields) {
        return new AccountKeyFactory()._fromObject(fields);
    }
    _fromRLP(rlp) {
        if (rlp == "0x80") { // special case without type prefix
            return this.fromObject({ type: index_js_2.AccountKeyType.Nil });
        }
        const type = (0, index_js_2.getTypePrefix)(rlp);
        if (!(0, index_js_2.isKlaytnAccountKeyType)(type)) {
            throw new Error("Not a Klaytn account key");
        }
        const ctor = this.lookup(type);
        const instance = new ctor();
        instance.setFieldsFromRLP(rlp);
        return instance;
    }
    static fromRLP(rlp) {
        return new AccountKeyFactory()._fromRLP(rlp);
    }
}
exports.AccountKeyFactory = AccountKeyFactory;
function parseAccountKey(rlp) {
    const key = AccountKeyFactory.fromRLP(rlp);
    const canonical = key.toObject();
    if (key.type == index_js_2.AccountKeyType.Nil || key.type == index_js_2.AccountKeyType.Legacy || key.type == index_js_2.AccountKeyType.Fail) {
        return { type: key.type };
    }
    if (key.type == index_js_2.AccountKeyType.Public) {
        return { type: key.type, key: canonical.key };
    }
    if (key.type == index_js_2.AccountKeyType.WeightedMultiSig) {
        return {
            type: key.type,
            threshold: index_js_2.HexStr.toNumber(canonical.threshold),
            // keys field of AccountKeyWeightedMultiSig already parse the RLP to object format
            keys: key.getField('keys'),
        };
    }
    if (key.type == index_js_2.AccountKeyType.RoleBased) {
        return {
            type: key.type,
            // keys field of AccountKeyRoleBased already parse the RLP to object format
            keys: key.getField('keys'),
        };
    }
    throw new Error(`Unknown AccountKeyType ${key.type}`);
}
