"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.TxTypeCancel = exports.TxTypeAccountUpdate = exports.TxTypeSmartContractExecution = exports.TxTypeSmartContractDeploy = exports.TxTypeValueTransferMemo = exports.TxTypeValueTransfer = void 0;
const index_js_1 = require("../field/index.js");
const index_js_2 = require("../util/index.js");
const abstract_js_1 = require("./abstract.js");
// https://docs.kaia.io/learn/transactions/basic#txtypevaluetransfer-
class TxTypeValueTransfer extends abstract_js_1.KlaytnTx {
    static type = index_js_2.TxType.ValueTransfer;
    static typeName = "TxTypeValueTransfer";
    static fieldTypes = {
        "type": index_js_1.FieldTypeUint8,
        "nonce": index_js_1.FieldTypeUint64,
        "gasPrice": index_js_1.FieldTypeUint256,
        "gasLimit": index_js_1.FieldTypeUint64,
        "to": index_js_1.FieldTypeAddress,
        "value": index_js_1.FieldTypeUint256,
        "from": index_js_1.FieldTypeAddress,
        "chainId": index_js_1.FieldTypeUint64,
        "txSignatures": index_js_1.FieldTypeSignatureTuples,
    };
    // SigRLP = encode([encode([type, nonce, gasPrice, gas, to, value, from]), chainid, 0, 0])
    sigRLP() {
        return this.encodeNestedRLP(["type", "nonce", "gasPrice", "gasLimit", "to", "value", "from"], ["chainId"]);
    }
    // TxHashRLP = type + encode([nonce, gasPrice, gas, to, value, from, txSignatures])
    txHashRLP() {
        return this.encodeTypePrefixedRLP(["nonce", "gasPrice", "gasLimit", "to", "value", "from", "txSignatures"]);
    }
    // TxHashRLP = type + encode([nonce, gasPrice, gas, to, value, from, txSignatures])
    setFieldsFromRLP(rlp) {
        this.decodeTypePrefixedRLP(rlp, ["nonce", "gasPrice", "gasLimit", "to", "value", "from", "txSignatures"]);
    }
}
exports.TxTypeValueTransfer = TxTypeValueTransfer;
// https://docs.kaia.io/learn/transactions/basic#txtypevaluetransfermemo-
class TxTypeValueTransferMemo extends abstract_js_1.KlaytnTx {
    static type = index_js_2.TxType.ValueTransferMemo;
    static typeName = "TxTypeValueTransferMemo";
    static fieldTypes = {
        "type": index_js_1.FieldTypeUint8,
        "nonce": index_js_1.FieldTypeUint64,
        "gasPrice": index_js_1.FieldTypeUint256,
        "gasLimit": index_js_1.FieldTypeUint64,
        "to": index_js_1.FieldTypeAddress,
        "value": index_js_1.FieldTypeUint256,
        "from": index_js_1.FieldTypeAddress,
        "data": index_js_1.FieldTypeBytes,
        "chainId": index_js_1.FieldTypeUint64,
        "txSignatures": index_js_1.FieldTypeSignatureTuples,
    };
    // SigRLP = encode([encode([type, nonce, gasPrice, gas, to, value, from, input]), chainid, 0, 0])
    sigRLP() {
        return this.encodeNestedRLP(["type", "nonce", "gasPrice", "gasLimit", "to", "value", "from", "data"], ["chainId"]);
    }
    // TxHashRLP = type + encode([nonce, gasPrice, gas, to, value, from, input, txSignatures])
    txHashRLP() {
        return this.encodeTypePrefixedRLP(["nonce", "gasPrice", "gasLimit", "to", "value", "from", "data", "txSignatures"]);
    }
    // TxHashRLP = type + encode([nonce, gasPrice, gas, to, value, from, input, txSignatures])
    setFieldsFromRLP(rlp) {
        this.decodeTypePrefixedRLP(rlp, ["nonce", "gasPrice", "gasLimit", "to", "value", "from", "data", "txSignatures"]);
    }
}
exports.TxTypeValueTransferMemo = TxTypeValueTransferMemo;
// https://docs.kaia.io/learn/transactions/basic#txtypesmartcontractdeploy-
class TxTypeSmartContractDeploy extends abstract_js_1.KlaytnTx {
    static type = index_js_2.TxType.SmartContractDeploy;
    static typeName = "TxTypeSmartContractDeploy";
    static fieldTypes = {
        "type": index_js_1.FieldTypeUint8,
        "nonce": index_js_1.FieldTypeUint64,
        "gasPrice": index_js_1.FieldTypeUint256,
        "gasLimit": index_js_1.FieldTypeUint64,
        "to": index_js_1.FieldTypeAddress,
        "value": index_js_1.FieldTypeUint256,
        "from": index_js_1.FieldTypeAddress,
        "data": index_js_1.FieldTypeBytes,
        "humanReadable": index_js_1.FieldTypeBool,
        "codeFormat": index_js_1.FieldTypeUint8,
        "chainId": index_js_1.FieldTypeUint64,
        "txSignatures": index_js_1.FieldTypeSignatureTuples,
    };
    // SigRLP = encode([encode([type, nonce, gasPrice, gas, to, value, from, input, humanReadable, codeFormat]), chainid, 0, 0])
    sigRLP() {
        return this.encodeNestedRLP(["type", "nonce", "gasPrice", "gasLimit", "to", "value", "from", "data", "humanReadable", "codeFormat"], ["chainId"]);
    }
    // TxHashRLP = type + encode([nonce, gasPrice, gas, to, value, from, input, humanReadable, codeFormat, txSignatures])
    txHashRLP() {
        return this.encodeTypePrefixedRLP(["nonce", "gasPrice", "gasLimit", "to", "value", "from", "data", "humanReadable", "codeFormat", "txSignatures"]);
    }
    // TxHashRLP = type + encode([nonce, gasPrice, gas, to, value, from, input, humanReadable, codeFormat, txSignatures])
    setFieldsFromRLP(rlp) {
        this.decodeTypePrefixedRLP(rlp, ["nonce", "gasPrice", "gasLimit", "to", "value", "from", "data", "humanReadable", "codeFormat", "txSignatures"]);
    }
}
exports.TxTypeSmartContractDeploy = TxTypeSmartContractDeploy;
// https://docs.kaia.io/learn/transactions/basic#txtypesmartcontractexecution-
class TxTypeSmartContractExecution extends abstract_js_1.KlaytnTx {
    static type = index_js_2.TxType.SmartContractExecution;
    static typeName = "TxTypeSmartContractExecution";
    static fieldTypes = {
        "type": index_js_1.FieldTypeUint8,
        "nonce": index_js_1.FieldTypeUint64,
        "gasPrice": index_js_1.FieldTypeUint256,
        "gasLimit": index_js_1.FieldTypeUint64,
        "to": index_js_1.FieldTypeAddress,
        "value": index_js_1.FieldTypeUint256,
        "from": index_js_1.FieldTypeAddress,
        "data": index_js_1.FieldTypeBytes,
        "chainId": index_js_1.FieldTypeUint64,
        "txSignatures": index_js_1.FieldTypeSignatureTuples,
    };
    // SigRLP = encode([encode([type, nonce, gasPrice, gas, to, value, from, input]), chainid, 0, 0])
    sigRLP() {
        return this.encodeNestedRLP(["type", "nonce", "gasPrice", "gasLimit", "to", "value", "from", "data"], ["chainId"]);
    }
    // TxHashRLP = type + encode([nonce, gasPrice, gas, to, value, from, input, txSignatures])
    txHashRLP() {
        return this.encodeTypePrefixedRLP(["nonce", "gasPrice", "gasLimit", "to", "value", "from", "data", "txSignatures"]);
    }
    // TxHashRLP = type + encode([nonce, gasPrice, gas, to, value, from, input, txSignatures])
    setFieldsFromRLP(rlp) {
        this.decodeTypePrefixedRLP(rlp, ["nonce", "gasPrice", "gasLimit", "to", "value", "from", "data", "txSignatures"]);
    }
}
exports.TxTypeSmartContractExecution = TxTypeSmartContractExecution;
// https://docs.kaia.io/learn/transactions/basic#txtypeaccountupdate-
class TxTypeAccountUpdate extends abstract_js_1.KlaytnTx {
    static type = index_js_2.TxType.AccountUpdate;
    static typeName = "TxTypeAccountUpdate";
    static fieldTypes = {
        "type": index_js_1.FieldTypeUint8,
        "nonce": index_js_1.FieldTypeUint64,
        "gasPrice": index_js_1.FieldTypeUint256,
        "gasLimit": index_js_1.FieldTypeUint64,
        "from": index_js_1.FieldTypeAddress,
        "key": index_js_1.FieldTypeAccountKey,
        "chainId": index_js_1.FieldTypeUint64,
        "txSignatures": index_js_1.FieldTypeSignatureTuples,
    };
    // SigRLP = encode([encode([type, nonce, gasPrice, gas, from, rlpEncodedKey]), chainid, 0, 0])
    sigRLP() {
        return this.encodeNestedRLP(["type", "nonce", "gasPrice", "gasLimit", "from", "key"], ["chainId"]);
    }
    // TxHashRLP = type + encode([nonce, gasPrice, gas, from, rlpEncodedKey, txSignatures])
    txHashRLP() {
        return this.encodeTypePrefixedRLP(["nonce", "gasPrice", "gasLimit", "from", "key", "txSignatures"]);
    }
    // TxHashRLP = type + encode([nonce, gasPrice, gas, from, rlpEncodedKey, txSignatures])
    setFieldsFromRLP(rlp) {
        this.decodeTypePrefixedRLP(rlp, ["nonce", "gasPrice", "gasLimit", "from", "key", "txSignatures"]);
    }
}
exports.TxTypeAccountUpdate = TxTypeAccountUpdate;
// https://docs.kaia.io/learn/transactions/basic#txtypecancel-
class TxTypeCancel extends abstract_js_1.KlaytnTx {
    static type = index_js_2.TxType.Cancel;
    static typeName = "TxTypeCancel";
    static fieldTypes = {
        "type": index_js_1.FieldTypeUint8,
        "nonce": index_js_1.FieldTypeUint64,
        "gasPrice": index_js_1.FieldTypeUint256,
        "gasLimit": index_js_1.FieldTypeUint64,
        "from": index_js_1.FieldTypeAddress,
        "chainId": index_js_1.FieldTypeUint64,
        "txSignatures": index_js_1.FieldTypeSignatureTuples,
    };
    // SigRLP = encode([encode([type, nonce, gasPrice, gas, from]), chainid, 0, 0])
    sigRLP() {
        return this.encodeNestedRLP(["type", "nonce", "gasPrice", "gasLimit", "from"], ["chainId"]);
    }
    // TxHashRLP = type + encode([nonce, gasPrice, gas, from, txSignatures])
    txHashRLP() {
        return this.encodeTypePrefixedRLP(["nonce", "gasPrice", "gasLimit", "from", "txSignatures"]);
    }
    // TxHashRLP = type + encode([nonce, gasPrice, gas, from, txSignatures])
    setFieldsFromRLP(rlp) {
        this.decodeTypePrefixedRLP(rlp, ["nonce", "gasPrice", "gasLimit", "from", "txSignatures"]);
    }
}
exports.TxTypeCancel = TxTypeCancel;
