"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.KlaytnTxFactory = void 0;
exports.parseTransaction = parseTransaction;
const bytes_1 = require("@ethersproject/bytes");
const keccak256_1 = require("@ethersproject/keccak256");
const ethers_1 = require("ethers");
const index_js_1 = require("../field/index.js");
const index_js_2 = require("../util/index.js");
const feedelegated_js_1 = require("./feedelegated.js");
const basic_js_1 = require("./basic.js");
const partialfeedelegated_js_1 = require("./partialfeedelegated.js");
class _KlaytnTxFactory extends index_js_1.FieldSetFactory {
    constructor() {
        const requiredFields = ["type", "chainId", "txSignatures"];
        super(requiredFields);
        this.add(basic_js_1.TxTypeValueTransfer);
        this.add(basic_js_1.TxTypeValueTransferMemo);
        this.add(basic_js_1.TxTypeSmartContractDeploy);
        this.add(basic_js_1.TxTypeSmartContractExecution);
        this.add(basic_js_1.TxTypeAccountUpdate);
        this.add(basic_js_1.TxTypeCancel);
        this.add(feedelegated_js_1.TxTypeFeeDelegatedValueTransfer);
        this.add(feedelegated_js_1.TxTypeFeeDelegatedValueTransferMemo);
        this.add(feedelegated_js_1.TxTypeFeeDelegatedSmartContractDeploy);
        this.add(feedelegated_js_1.TxTypeFeeDelegatedSmartContractExecution);
        this.add(feedelegated_js_1.TxTypeFeeDelegatedAccountUpdate);
        this.add(feedelegated_js_1.TxTypeFeeDelegatedCancel);
        this.add(partialfeedelegated_js_1.TxTypeFeeDelegatedValueTransferWithRatio);
        this.add(partialfeedelegated_js_1.TxTypeFeeDelegatedValueTransferMemoWithRatio);
        this.add(partialfeedelegated_js_1.TxTypeFeeDelegatedSmartContractDeployWithRatio);
        this.add(partialfeedelegated_js_1.TxTypeFeeDelegatedSmartContractExecutionWithRatio);
        this.add(partialfeedelegated_js_1.TxTypeFeeDelegatedAccountUpdateWithRatio);
        this.add(partialfeedelegated_js_1.TxTypeFeeDelegatedCancelWithRatio);
    }
    fromObject(fields) {
        // Alias input -> data for compatiblity
        if (fields.input) {
            fields.data = fields.input;
        }
        // In TxTypeSmartContractDeploy, force 'to' = 0x for compatibility
        if (index_js_2.HexStr.fromNumber(fields.type) == index_js_2.HexStr.fromNumber(index_js_2.TxType.SmartContractDeploy) ||
            index_js_2.HexStr.fromNumber(fields.type) == index_js_2.HexStr.fromNumber(index_js_2.TxType.FeeDelegatedSmartContractDeploy) ||
            index_js_2.HexStr.fromNumber(fields.type) == index_js_2.HexStr.fromNumber(index_js_2.TxType.FeeDelegatedSmartContractDeployWithRatio)) {
            fields.to = "0x";
        }
        return super.fromObject(fields);
    }
    fromRLP(rlp) {
        const type = (0, index_js_2.getTypePrefix)(rlp);
        if (!(0, index_js_2.isKlaytnTxType)(type)) {
            throw new Error("Not a Klaytn raw transaction");
        }
        const ctor = this.lookup(type);
        const instance = new ctor();
        instance.setFieldsFromRLP(rlp);
        return instance;
    }
}
exports.KlaytnTxFactory = new _KlaytnTxFactory();
/* eslint-disable no-multi-spaces */
function parseTransaction(rlp) {
    const type = (0, index_js_2.getTypePrefix)(rlp);
    if (!(0, index_js_2.isKlaytnTxType)(type)) {
        const tx = ethers_1.Transaction.from(rlp);
        // eip1559
        const parsedTx = {
            // ...tx,
            from: tx.from ?? undefined,
            to: tx.to ?? undefined,
            nonce: tx.nonce,
            data: tx.data,
            r: tx.signature?.r,
            s: tx.signature?.s,
            v: tx.signature?.v ?? undefined,
            hash: tx.hash ?? undefined,
            accessList: tx.accessList ?? undefined,
            chainId: Number(tx.chainId),
            type: tx?.type,
            // Convert BigNumber to hex string
            gasLimit: (0, bytes_1.hexValue)(tx.gasLimit),
            gasPrice: tx.gasPrice ? (0, bytes_1.hexValue)(tx.gasPrice) : undefined,
            value: (0, bytes_1.hexValue)(tx.value),
            maxPriorityFeePerGas: tx.maxPriorityFeePerGas ? (0, bytes_1.hexValue)(tx.maxPriorityFeePerGas) : undefined,
            maxFeePerGas: tx.maxFeePerGas ? (0, bytes_1.hexValue)(tx.maxFeePerGas) : undefined,
        };
        if (typeof parsedTx.v === 'number') {
            // convert v to KAIA compatible
            parsedTx.v = parsedTx.v === 27 ? 0 : 1;
        }
        // Clean up 'explicit undefined' fields
        Object.entries(parsedTx).forEach(([key, value]) => {
            if (value === undefined) {
                delete parsedTx[key];
            }
        });
        return parsedTx;
    }
    else {
        const tx = exports.KlaytnTxFactory.fromRLP(rlp).toObject();
        try {
            const parsedTx = {
                hash: (0, keccak256_1.keccak256)(rlp),
                to: tx.to ? index_js_2.HexStr.toAddress(tx.to) : undefined,
                from: tx.from ? index_js_2.HexStr.toAddress(tx.from) : undefined,
                nonce: index_js_2.HexStr.toNumber(tx.nonce),
                gasLimit: (0, bytes_1.hexValue)(tx.gasLimit),
                gasPrice: (0, bytes_1.hexValue)(tx.gasPrice),
                data: tx.data ?? "0x",
                value: (0, bytes_1.hexValue)(tx.value ?? 0),
                chainId: tx.chainId ? index_js_2.HexStr.toNumber(tx.chainId) : undefined,
                type: tx.type ? index_js_2.HexStr.toNumber(tx.type) : null,
                accessList: tx.accessList,
                key: tx.key,
                humanReadable: tx.humanReadable ? index_js_2.HexStr.toBoolean(tx.humanReadable) : undefined,
                codeFormat: tx.codeFormat ? index_js_2.HexStr.toNumber(tx.codeFormat) : undefined,
                feePayer: tx.feePayer ? index_js_2.HexStr.toAddress(tx.feePayer) : undefined,
                txSignatures: tx.txSignatures,
                feePayerSignatures: tx.feePayerSignatures,
                feeRatio: tx.feeRatio ? index_js_2.HexStr.toNumber(tx.feeRatio) : undefined,
            };
            // Clean up 'explicit undefined' fields
            Object.entries(parsedTx).forEach(([key, value]) => {
                if (value === undefined) {
                    delete parsedTx[key];
                }
            });
            return parsedTx;
        }
        catch (e) {
            // In case conversion fails, return the original tx object.
            return tx;
        }
    }
}
/* eslint-enable no-multi-spaces */
