"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.TxTypeFeeDelegatedCancel = exports.TxTypeFeeDelegatedAccountUpdate = exports.TxTypeFeeDelegatedSmartContractExecution = exports.TxTypeFeeDelegatedSmartContractDeploy = exports.TxTypeFeeDelegatedValueTransferMemo = exports.TxTypeFeeDelegatedValueTransfer = void 0;
const index_js_1 = require("../field/index.js");
const index_js_2 = require("../util/index.js");
const abstract_js_1 = require("./abstract.js");
// https://docs.kaia.io/learn/transactions/fee-delegation#txtypefeedelegatedvaluetransfer-
class TxTypeFeeDelegatedValueTransfer extends abstract_js_1.KlaytnTx {
    static type = index_js_2.TxType.FeeDelegatedValueTransfer;
    static typeName = "TxTypeFeeDelegatedValueTransfer";
    static fieldTypes = {
        "type": index_js_1.FieldTypeUint8,
        "nonce": index_js_1.FieldTypeUint64,
        "gasPrice": index_js_1.FieldTypeUint256,
        "gasLimit": index_js_1.FieldTypeUint64,
        "to": index_js_1.FieldTypeAddress,
        "value": index_js_1.FieldTypeUint256,
        "from": index_js_1.FieldTypeAddress,
        "chainId": index_js_1.FieldTypeUint64,
        "txSignatures": index_js_1.FieldTypeSignatureTuples,
        "feePayer": index_js_1.FieldTypeAddress,
        "feePayerSignatures": index_js_1.FieldTypeSignatureTuples,
    };
    // SigRLP = encode([encode([type, nonce, gasPrice, gas, to, value, from]), chainid, 0, 0])
    sigRLP() {
        return this.encodeNestedRLP(["type", "nonce", "gasPrice", "gasLimit", "to", "value", "from"], ["chainId"]);
    }
    // SigFeePayerRLP = encode([encode([type, nonce, gasPrice, gas, to, value, from]), feePayer, chainid, 0, 0 ])
    sigFeePayerRLP() {
        return this.encodeNestedRLP(["type", "nonce", "gasPrice", "gasLimit", "to", "value", "from"], ["feePayer", "chainId"]);
    }
    // SenderTxHashRLP = type + encode([nonce, gasPrice, gas, to, value, from, txSignatures])
    senderTxHashRLP() {
        return this.encodeTypePrefixedRLP(["nonce", "gasPrice", "gasLimit", "to", "value", "from", "txSignatures"]);
    }
    // TxHashRLP = type + encode([nonce, gasPrice, gas, to, value, from, txSignatures, feePayer, feePayerSignatures])
    txHashRLP() {
        return this.encodeTypePrefixedRLP(["nonce", "gasPrice", "gasLimit", "to", "value", "from", "txSignatures", "feePayer", "feePayerSignatures"]);
    }
    // SenderTxHashRLP = type + encode([nonce, gasPrice, gas, to, value, from, txSignatures])
    // TxHashRLP       = type + encode([nonce, gasPrice, gas, to, value, from, txSignatures, feePayer, feePayerSignatures])
    setFieldsFromRLP(rlp) {
        this.decodeTypePrefixedVarlenRLP(rlp, ["nonce", "gasPrice", "gasLimit", "to", "value", "from", "txSignatures"], ["nonce", "gasPrice", "gasLimit", "to", "value", "from", "txSignatures", "feePayer", "feePayerSignatures"]);
    }
}
exports.TxTypeFeeDelegatedValueTransfer = TxTypeFeeDelegatedValueTransfer;
// https://docs.kaia.io/learn/transactions/fee-delegation#txtypefeedelegatedvaluetransfermemo-
class TxTypeFeeDelegatedValueTransferMemo extends abstract_js_1.KlaytnTx {
    static type = index_js_2.TxType.FeeDelegatedValueTransferMemo;
    static typeName = "TxTypeFeeDelegatedValueTransferMemo";
    static fieldTypes = {
        "type": index_js_1.FieldTypeUint8,
        "nonce": index_js_1.FieldTypeUint64,
        "gasPrice": index_js_1.FieldTypeUint256,
        "gasLimit": index_js_1.FieldTypeUint64,
        "to": index_js_1.FieldTypeAddress,
        "value": index_js_1.FieldTypeUint256,
        "from": index_js_1.FieldTypeAddress,
        "data": index_js_1.FieldTypeBytes,
        "chainId": index_js_1.FieldTypeUint64,
        "txSignatures": index_js_1.FieldTypeSignatureTuples,
        "feePayer": index_js_1.FieldTypeAddress,
        "feePayerSignatures": index_js_1.FieldTypeSignatureTuples,
    };
    // SigRLP = encode([encode([type, nonce, gasPrice, gas, to, value, from, input]), chainid, 0, 0])
    sigRLP() {
        return this.encodeNestedRLP(["type", "nonce", "gasPrice", "gasLimit", "to", "value", "from", "data"], ["chainId"]);
    }
    // SigFeePayerRLP = encode([encode([type, nonce, gasPrice, gas, to, value, from, input]), feePayer, chainid, 0, 0])
    sigFeePayerRLP() {
        return this.encodeNestedRLP(["type", "nonce", "gasPrice", "gasLimit", "to", "value", "from", "data"], ["feePayer", "chainId"]);
    }
    // SenderTxHashRLP = type + encode([nonce, gasPrice, gas, to, value, from, input, txSignatures])
    senderTxHashRLP() {
        return this.encodeTypePrefixedRLP(["nonce", "gasPrice", "gasLimit", "to", "value", "from", "data", "txSignatures"]);
    }
    // TxHashRLP = type + encode([nonce, gasPrice, gas, to, value, from, input, txSignatures, feePayer, feePayerSignatures])
    txHashRLP() {
        return this.encodeTypePrefixedRLP(["nonce", "gasPrice", "gasLimit", "to", "value", "from", "data", "txSignatures", "feePayer", "feePayerSignatures"]);
    }
    // SenderTxHashRLP = type + encode([nonce, gasPrice, gas, to, value, from, input, txSignatures])
    // TxHashRLP = type + encode([nonce, gasPrice, gas, to, value, from, input, txSignatures, feePayer, feePayerSignatures])
    setFieldsFromRLP(rlp) {
        this.decodeTypePrefixedVarlenRLP(rlp, ["nonce", "gasPrice", "gasLimit", "to", "value", "from", "data", "txSignatures"], ["nonce", "gasPrice", "gasLimit", "to", "value", "from", "data", "txSignatures", "feePayer", "feePayerSignatures"]);
    }
}
exports.TxTypeFeeDelegatedValueTransferMemo = TxTypeFeeDelegatedValueTransferMemo;
// https://docs.kaia.io/learn/transactions/fee-delegation#txtypefeedelegatedsmartcontractdeploy-
class TxTypeFeeDelegatedSmartContractDeploy extends abstract_js_1.KlaytnTx {
    static type = index_js_2.TxType.FeeDelegatedSmartContractDeploy;
    static typeName = "TxTypeFeeDelegatedSmartContractDeploy";
    static fieldTypes = {
        "type": index_js_1.FieldTypeUint8,
        "nonce": index_js_1.FieldTypeUint64,
        "gasPrice": index_js_1.FieldTypeUint256,
        "gasLimit": index_js_1.FieldTypeUint64,
        "to": index_js_1.FieldTypeAddress,
        "value": index_js_1.FieldTypeUint256,
        "from": index_js_1.FieldTypeAddress,
        "data": index_js_1.FieldTypeBytes,
        "humanReadable": index_js_1.FieldTypeBool,
        "codeFormat": index_js_1.FieldTypeUint8,
        "chainId": index_js_1.FieldTypeUint64,
        "txSignatures": index_js_1.FieldTypeSignatureTuples,
        "feePayer": index_js_1.FieldTypeAddress,
        "feePayerSignatures": index_js_1.FieldTypeSignatureTuples,
    };
    // SigRLP = encode([encode([type, nonce, gasPrice, gas, to, value, from, input, humanReadable, codeFormat]), chainid, 0, 0])
    sigRLP() {
        return this.encodeNestedRLP(["type", "nonce", "gasPrice", "gasLimit", "to", "value", "from", "data", "humanReadable", "codeFormat"], ["chainId"]);
    }
    // SigFeePayerRLP = encode([encode([type, nonce, gasPrice, gas, to, value, from, input, humanReadable, codeFormat]), feePayer, chainid, 0, 0])
    sigFeePayerRLP() {
        return this.encodeNestedRLP(["type", "nonce", "gasPrice", "gasLimit", "to", "value", "from", "data", "humanReadable", "codeFormat"], ["feePayer", "chainId"]);
    }
    // SenderTxHashRLP = type + encode([nonce, gasPrice, gas, to, value, from, input, humanReadable, codeFormat, txSignatures])
    senderTxHashRLP() {
        return this.encodeTypePrefixedRLP(["nonce", "gasPrice", "gasLimit", "to", "value", "from", "data", "humanReadable", "codeFormat", "txSignatures"]);
    }
    // TxHashRLP = type + encode([nonce, gasPrice, gas, to, value, from, input, humanReadable, codeFormat, txSignatures, feePayer, feePayerSignatures])
    txHashRLP() {
        return this.encodeTypePrefixedRLP(["nonce", "gasPrice", "gasLimit", "to", "value", "from", "data", "humanReadable", "codeFormat", "txSignatures", "feePayer", "feePayerSignatures"]);
    }
    // SenderTxHashRLP = type + encode([nonce, gasPrice, gas, to, value, from, input, humanReadable, codeFormat, txSignatures])
    // TxHashRLP = type + encode([nonce, gasPrice, gas, to, value, from, input, humanReadable, codeFormat, txSignatures, feePayer, feePayerSignatures])
    setFieldsFromRLP(rlp) {
        this.decodeTypePrefixedVarlenRLP(rlp, ["nonce", "gasPrice", "gasLimit", "to", "value", "from", "data", "humanReadable", "codeFormat", "txSignatures"], ["nonce", "gasPrice", "gasLimit", "to", "value", "from", "data", "humanReadable", "codeFormat", "txSignatures", "feePayer", "feePayerSignatures"]);
    }
}
exports.TxTypeFeeDelegatedSmartContractDeploy = TxTypeFeeDelegatedSmartContractDeploy;
// https://docs.kaia.io/learn/transactions/fee-delegation#txtypefeedelegatedsmartcontractexecution-
class TxTypeFeeDelegatedSmartContractExecution extends abstract_js_1.KlaytnTx {
    static type = index_js_2.TxType.FeeDelegatedSmartContractExecution;
    static typeName = "TxTypeFeeDelegatedSmartContractExecution";
    static fieldTypes = {
        "type": index_js_1.FieldTypeUint8,
        "nonce": index_js_1.FieldTypeUint64,
        "gasPrice": index_js_1.FieldTypeUint256,
        "gasLimit": index_js_1.FieldTypeUint64,
        "to": index_js_1.FieldTypeAddress,
        "value": index_js_1.FieldTypeUint256,
        "from": index_js_1.FieldTypeAddress,
        "data": index_js_1.FieldTypeBytes,
        "chainId": index_js_1.FieldTypeUint64,
        "txSignatures": index_js_1.FieldTypeSignatureTuples,
        "feePayer": index_js_1.FieldTypeAddress,
        "feePayerSignatures": index_js_1.FieldTypeSignatureTuples,
    };
    // SigRLP = encode([encode([type, nonce, gasPrice, gas, to, value, from, input]), chainid, 0, 0])
    sigRLP() {
        return this.encodeNestedRLP(["type", "nonce", "gasPrice", "gasLimit", "to", "value", "from", "data"], ["chainId"]);
    }
    // SigFeePayerRLP = encode([encode([type, nonce, gasPrice, gas, to, value, from, input]), feePayer, chainid, 0, 0])
    sigFeePayerRLP() {
        return this.encodeNestedRLP(["type", "nonce", "gasPrice", "gasLimit", "to", "value", "from", "data"], ["feePayer", "chainId"]);
    }
    // SenderTxHashRLP = type + encode([nonce, gasPrice, gas, to, value, from, input, txSignatures])
    senderTxHashRLP() {
        return this.encodeTypePrefixedRLP(["nonce", "gasPrice", "gasLimit", "to", "value", "from", "data", "txSignatures"]);
    }
    // TxHashRLP = type + encode([nonce, gasPrice, gas, to, value, from, input, txSignatures, feePayer, feePayerSignatures])
    txHashRLP() {
        return this.encodeTypePrefixedRLP(["nonce", "gasPrice", "gasLimit", "to", "value", "from", "data", "txSignatures", "feePayer", "feePayerSignatures"]);
    }
    // SenderTxHashRLP = type + encode([nonce, gasPrice, gas, to, value, from, input, txSignatures])
    // TxHashRLP = type + encode([nonce, gasPrice, gas, to, value, from, input, txSignatures, feePayer, feePayerSignatures])
    setFieldsFromRLP(rlp) {
        this.decodeTypePrefixedVarlenRLP(rlp, ["nonce", "gasPrice", "gasLimit", "to", "value", "from", "data", "txSignatures"], ["nonce", "gasPrice", "gasLimit", "to", "value", "from", "data", "txSignatures", "feePayer", "feePayerSignatures"]);
    }
}
exports.TxTypeFeeDelegatedSmartContractExecution = TxTypeFeeDelegatedSmartContractExecution;
// https://docs.kaia.io/learn/transactions/fee-delegation#txtypefeedelegatedaccountupdate-
class TxTypeFeeDelegatedAccountUpdate extends abstract_js_1.KlaytnTx {
    static type = index_js_2.TxType.FeeDelegatedAccountUpdate;
    static typeName = "TxTypeFeeDelegatedAccountUpdate";
    static fieldTypes = {
        "type": index_js_1.FieldTypeUint8,
        "nonce": index_js_1.FieldTypeUint64,
        "gasPrice": index_js_1.FieldTypeUint256,
        "gasLimit": index_js_1.FieldTypeUint64,
        "from": index_js_1.FieldTypeAddress,
        "key": index_js_1.FieldTypeAccountKey,
        "chainId": index_js_1.FieldTypeUint64,
        "txSignatures": index_js_1.FieldTypeSignatureTuples,
        "feePayer": index_js_1.FieldTypeAddress,
        "feePayerSignatures": index_js_1.FieldTypeSignatureTuples,
    };
    // SigRLP = encode([encode([type, nonce, gasPrice, gas, from, rlpEncodedKey]), chainid, 0, 0])
    sigRLP() {
        return this.encodeNestedRLP(["type", "nonce", "gasPrice", "gasLimit", "from", "key"], ["chainId"]);
    }
    // SigFeePayerRLP = encode([encode([type, nonce, gasPrice, gas, from, rlpEncodedKey]), feePayer, chainid, 0, 0])
    sigFeePayerRLP() {
        return this.encodeNestedRLP(["type", "nonce", "gasPrice", "gasLimit", "from", "key"], ["feePayer", "chainId"]);
    }
    // SenderTxHashRLP = type + encode([nonce, gasPrice, gas, from, rlpEncodedKey, txSignatures])
    senderTxHashRLP() {
        return this.encodeTypePrefixedRLP(["nonce", "gasPrice", "gasLimit", "from", "key", "txSignatures"]);
    }
    // TxHashRLP = type + encode([nonce, gasPrice, gas, from, rlpEncodedKey, txSignatures, feePayer, feePayerSignatures])
    txHashRLP() {
        return this.encodeTypePrefixedRLP(["nonce", "gasPrice", "gasLimit", "from", "key", "txSignatures", "feePayer", "feePayerSignatures"]);
    }
    // SenderTxHashRLP = type + encode([nonce, gasPrice, gas, from, rlpEncodedKey, txSignatures])
    // TxHashRLP = type + encode([nonce, gasPrice, gas, from, rlpEncodedKey, txSignatures, feePayer, feePayerSignatures])
    setFieldsFromRLP(rlp) {
        this.decodeTypePrefixedVarlenRLP(rlp, ["nonce", "gasPrice", "gasLimit", "from", "key", "txSignatures"], ["nonce", "gasPrice", "gasLimit", "from", "key", "txSignatures", "feePayer", "feePayerSignatures"]);
    }
}
exports.TxTypeFeeDelegatedAccountUpdate = TxTypeFeeDelegatedAccountUpdate;
// https://docs.kaia.io/learn/transactions/fee-delegation#txtypefeedelegatedcancel-
class TxTypeFeeDelegatedCancel extends abstract_js_1.KlaytnTx {
    static type = index_js_2.TxType.FeeDelegatedCancel;
    static typeName = "TxTypeFeeDelegatedCancel";
    static fieldTypes = {
        "type": index_js_1.FieldTypeUint8,
        "nonce": index_js_1.FieldTypeUint64,
        "gasPrice": index_js_1.FieldTypeUint256,
        "gasLimit": index_js_1.FieldTypeUint64,
        "from": index_js_1.FieldTypeAddress,
        "chainId": index_js_1.FieldTypeUint64,
        "txSignatures": index_js_1.FieldTypeSignatureTuples,
        "feePayer": index_js_1.FieldTypeAddress,
        "feePayerSignatures": index_js_1.FieldTypeSignatureTuples,
    };
    // SigRLP = encode([encode([type, nonce, gasPrice, gas, from]), chainid, 0, 0])
    sigRLP() {
        return this.encodeNestedRLP(["type", "nonce", "gasPrice", "gasLimit", "from"], ["chainId"]);
    }
    // SigFeePayerRLP = encode([encode([type, nonce, gasPrice, gas, from]), feePayer, chainid, 0, 0])
    sigFeePayerRLP() {
        return this.encodeNestedRLP(["type", "nonce", "gasPrice", "gasLimit", "from"], ["feePayer", "chainId"]);
    }
    // SenderTxHashRLP = type + encode([nonce, gasPrice, gas, from, txSignatures])
    senderTxHashRLP() {
        return this.encodeTypePrefixedRLP(["nonce", "gasPrice", "gasLimit", "from", "txSignatures"]);
    }
    // TxHashRLP = type + encode([nonce, gasPrice, gas, from, txSignatures, feePayer, feePayerSignatures])
    txHashRLP() {
        return this.encodeTypePrefixedRLP(["nonce", "gasPrice", "gasLimit", "from", "txSignatures", "feePayer", "feePayerSignatures"]);
    }
    // SenderTxHashRLP = type + encode([nonce, gasPrice, gas, from, txSignatures])
    // TxHashRLP = type + encode([nonce, gasPrice, gas, from, txSignatures, feePayer, feePayerSignatures])
    setFieldsFromRLP(rlp) {
        this.decodeTypePrefixedVarlenRLP(rlp, ["nonce", "gasPrice", "gasLimit", "from", "txSignatures"], ["nonce", "gasPrice", "gasLimit", "from", "txSignatures", "feePayer", "feePayerSignatures"]);
    }
}
exports.TxTypeFeeDelegatedCancel = TxTypeFeeDelegatedCancel;
