"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.TxTypeFeeDelegatedCancelWithRatio = exports.TxTypeFeeDelegatedAccountUpdateWithRatio = exports.TxTypeFeeDelegatedSmartContractExecutionWithRatio = exports.TxTypeFeeDelegatedSmartContractDeployWithRatio = exports.TxTypeFeeDelegatedValueTransferMemoWithRatio = exports.TxTypeFeeDelegatedValueTransferWithRatio = void 0;
const index_js_1 = require("../field/index.js");
const index_js_2 = require("../util/index.js");
const abstract_js_1 = require("./abstract.js");
// https://docs.kaia.io/learn/transactions/partial-fee-delegation#txtypefeedelegatedvaluetransferwithratio-
class TxTypeFeeDelegatedValueTransferWithRatio extends abstract_js_1.KlaytnTx {
    static type = index_js_2.TxType.FeeDelegatedValueTransferWithRatio;
    static typeName = "TxTypeFeeDelegatedValueTransferWithRatio";
    static fieldTypes = {
        "type": index_js_1.FieldTypeUint8,
        "nonce": index_js_1.FieldTypeUint64,
        "gasPrice": index_js_1.FieldTypeUint256,
        "gasLimit": index_js_1.FieldTypeUint64,
        "to": index_js_1.FieldTypeAddress,
        "value": index_js_1.FieldTypeUint256,
        "from": index_js_1.FieldTypeAddress,
        "feeRatio": index_js_1.FieldTypeUint8,
        "chainId": index_js_1.FieldTypeUint64,
        "txSignatures": index_js_1.FieldTypeSignatureTuples,
        "feePayer": index_js_1.FieldTypeAddress,
        "feePayerSignatures": index_js_1.FieldTypeSignatureTuples,
    };
    // SigRLP = encode([encode([type, nonce, gasPrice, gas, to, value, from, feeRatio]), chainid, 0, 0])
    sigRLP() {
        return this.encodeNestedRLP(["type", "nonce", "gasPrice", "gasLimit", "to", "value", "from", "feeRatio"], ["chainId"]);
    }
    // SigFeePayerRLP = encode([ encode([type, nonce, gasPrice, gas, to, value, from, feeRatio]), feePayer, chainid, 0, 0 ])
    sigFeePayerRLP() {
        return this.encodeNestedRLP(["type", "nonce", "gasPrice", "gasLimit", "to", "value", "from", "feeRatio"], ["feePayer", "chainId"]);
    }
    // SenderTxHashRLP = type + encode([nonce, gasPrice, gas, to, value, from, feeRatio, txSignatures])
    senderTxHashRLP() {
        return this.encodeTypePrefixedRLP(["nonce", "gasPrice", "gasLimit", "to", "value", "from", "feeRatio", "txSignatures"]);
    }
    // TxHashRLP = type + encode([nonce, gasPrice, gas, to, value, from, feeRatio, txSignatures, feePayer, feePayerSignatures])
    txHashRLP() {
        return this.encodeTypePrefixedRLP(["nonce", "gasPrice", "gasLimit", "to", "value", "from", "feeRatio", "txSignatures", "feePayer", "feePayerSignatures"]);
    }
    // SenderTxHashRLP = type + encode([nonce, gasPrice, gas, to, value, from, feeRatio, txSignatures])
    // TxHashRLP = type + encode([nonce, gasPrice, gas, to, value, from, feeRatio, txSignatures, feePayer, feePayerSignatures])
    setFieldsFromRLP(rlp) {
        this.decodeTypePrefixedVarlenRLP(rlp, ["nonce", "gasPrice", "gasLimit", "to", "value", "from", "feeRatio", "txSignatures"], ["nonce", "gasPrice", "gasLimit", "to", "value", "from", "feeRatio", "txSignatures", "feePayer", "feePayerSignatures"]);
    }
}
exports.TxTypeFeeDelegatedValueTransferWithRatio = TxTypeFeeDelegatedValueTransferWithRatio;
// https://docs.kaia.io/learn/transactions/partial-fee-delegation#txtypefeedelegatedvaluetransfermemowithratio-
class TxTypeFeeDelegatedValueTransferMemoWithRatio extends abstract_js_1.KlaytnTx {
    static type = index_js_2.TxType.FeeDelegatedValueTransferMemoWithRatio;
    static typeName = "TxTypeFeeDelegatedValueTransferMemoWithRatio";
    static fieldTypes = {
        "type": index_js_1.FieldTypeUint8,
        "nonce": index_js_1.FieldTypeUint64,
        "gasPrice": index_js_1.FieldTypeUint256,
        "gasLimit": index_js_1.FieldTypeUint64,
        "to": index_js_1.FieldTypeAddress,
        "value": index_js_1.FieldTypeUint256,
        "from": index_js_1.FieldTypeAddress,
        "data": index_js_1.FieldTypeBytes,
        "feeRatio": index_js_1.FieldTypeUint8,
        "chainId": index_js_1.FieldTypeUint64,
        "txSignatures": index_js_1.FieldTypeSignatureTuples,
        "feePayer": index_js_1.FieldTypeAddress,
        "feePayerSignatures": index_js_1.FieldTypeSignatureTuples,
    };
    // SigRLP = encode([encode([type, nonce, gasPrice, gas, to, value, from, input, feeRatio]), chainid, 0, 0])
    sigRLP() {
        return this.encodeNestedRLP(["type", "nonce", "gasPrice", "gasLimit", "to", "value", "from", "data", "feeRatio"], ["chainId"]);
    }
    // SigFeePayerRLP = encode([encode([type, nonce, gasPrice, gas, to, value, from, input, feeRatio]), feePayer, chainid, 0, 0])
    sigFeePayerRLP() {
        return this.encodeNestedRLP(["type", "nonce", "gasPrice", "gasLimit", "to", "value", "from", "data", "feeRatio"], ["feePayer", "chainId"]);
    }
    // SenderTxHashRLP = type + encode([nonce, gasPrice, gas, to, value, from, input, feeRatio, txSignatures])
    senderTxHashRLP() {
        return this.encodeTypePrefixedRLP(["nonce", "gasPrice", "gasLimit", "to", "value", "from", "data", "feeRatio", "txSignatures"]);
    }
    // TxHashRLP = type + encode([nonce, gasPrice, gas, to, value, from, input, feeRatio, txSignatures, feePayer, feePayerSignatures])
    txHashRLP() {
        return this.encodeTypePrefixedRLP(["nonce", "gasPrice", "gasLimit", "to", "value", "from", "data", "feeRatio", "txSignatures", "feePayer", "feePayerSignatures"]);
    }
    // SenderTxHashRLP = type + encode([nonce, gasPrice, gas, to, value, from, input, feeRatio, txSignatures])
    // TxHashRLP = type + encode([nonce, gasPrice, gas, to, value, from, input, feeRatio, txSignatures, feePayer, feePayerSignatures])
    setFieldsFromRLP(rlp) {
        this.decodeTypePrefixedVarlenRLP(rlp, ["nonce", "gasPrice", "gasLimit", "to", "value", "from", "data", "feeRatio", "txSignatures"], ["nonce", "gasPrice", "gasLimit", "to", "value", "from", "data", "feeRatio", "txSignatures", "feePayer", "feePayerSignatures"]);
    }
}
exports.TxTypeFeeDelegatedValueTransferMemoWithRatio = TxTypeFeeDelegatedValueTransferMemoWithRatio;
// https://docs.kaia.io/learn/transactions/partial-fee-delegation#txtypefeedelegatedsmartcontractdeploywithratio-
class TxTypeFeeDelegatedSmartContractDeployWithRatio extends abstract_js_1.KlaytnTx {
    static type = index_js_2.TxType.FeeDelegatedSmartContractDeployWithRatio;
    static typeName = "TxTypeFeeDelegatedSmartContractDeployWithRatio";
    static fieldTypes = {
        "type": index_js_1.FieldTypeUint8,
        "nonce": index_js_1.FieldTypeUint64,
        "gasPrice": index_js_1.FieldTypeUint256,
        "gasLimit": index_js_1.FieldTypeUint64,
        "to": index_js_1.FieldTypeAddress,
        "value": index_js_1.FieldTypeUint256,
        "from": index_js_1.FieldTypeAddress,
        "data": index_js_1.FieldTypeBytes,
        "humanReadable": index_js_1.FieldTypeBool,
        "feeRatio": index_js_1.FieldTypeUint8,
        "codeFormat": index_js_1.FieldTypeUint8,
        "chainId": index_js_1.FieldTypeUint64,
        "txSignatures": index_js_1.FieldTypeSignatureTuples,
        "feePayer": index_js_1.FieldTypeAddress,
        "feePayerSignatures": index_js_1.FieldTypeSignatureTuples,
    };
    // SigRLP = encode([encode([type, nonce, gasPrice, gas, to, value, from, input, humanReadable, feeRatio, codeFormat]), chainid, 0, 0])
    sigRLP() {
        return this.encodeNestedRLP(["type", "nonce", "gasPrice", "gasLimit", "to", "value", "from", "data", "humanReadable", "feeRatio", "codeFormat"], ["chainId"]);
    }
    // SigFeePayerRLP = encode([encode([type, nonce, gasPrice, gas, to, value, from, input, humanReadable, feeRatio, codeFormat]), feePayer, chainid, 0, 0])
    sigFeePayerRLP() {
        return this.encodeNestedRLP(["type", "nonce", "gasPrice", "gasLimit", "to", "value", "from", "data", "humanReadable", "feeRatio", "codeFormat"], ["feePayer", "chainId"]);
    }
    // SenderTxHashRLP = type + encode([nonce, gasPrice, gas, to, value, from, input, humanReadable, feeRatio, codeFormat, txSignatures])
    senderTxHashRLP() {
        return this.encodeTypePrefixedRLP(["nonce", "gasPrice", "gasLimit", "to", "value", "from", "data", "humanReadable", "feeRatio", "codeFormat", "txSignatures"]);
    }
    // TxHashRLP = type + encode([nonce, gasPrice, gas, to, value, from, input, humanReadable, feeRatio, codeFormat, txSignatures, feePayer, feePayerSignatures])
    txHashRLP() {
        return this.encodeTypePrefixedRLP(["nonce", "gasPrice", "gasLimit", "to", "value", "from", "data", "humanReadable", "feeRatio", "codeFormat", "txSignatures", "feePayer", "feePayerSignatures"]);
    }
    // SenderTxHashRLP = type + encode([nonce, gasPrice, gas, to, value, from, input, humanReadable, feeRatio, codeFormat, txSignatures])
    // TxHashRLP = type + encode([nonce, gasPrice, gas, to, value, from, input, humanReadable, feeRatio, codeFormat, txSignatures, feePayer, feePayerSignatures])
    setFieldsFromRLP(rlp) {
        this.decodeTypePrefixedVarlenRLP(rlp, ["nonce", "gasPrice", "gasLimit", "to", "value", "from", "data", "humanReadable", "feeRatio", "codeFormat", "txSignatures"], ["nonce", "gasPrice", "gasLimit", "to", "value", "from", "data", "humanReadable", "feeRatio", "codeFormat", "txSignatures", "feePayer", "feePayerSignatures"]);
    }
}
exports.TxTypeFeeDelegatedSmartContractDeployWithRatio = TxTypeFeeDelegatedSmartContractDeployWithRatio;
// https://docs.kaia.io/learn/transactions/partial-fee-delegation#txtypefeedelegatedsmartcontractexecutionwithratio-
class TxTypeFeeDelegatedSmartContractExecutionWithRatio extends abstract_js_1.KlaytnTx {
    static type = index_js_2.TxType.FeeDelegatedSmartContractExecutionWithRatio;
    static typeName = "TxTypeFeeDelegatedSmartContractExecutionWithRatio";
    static fieldTypes = {
        "type": index_js_1.FieldTypeUint8,
        "nonce": index_js_1.FieldTypeUint64,
        "gasPrice": index_js_1.FieldTypeUint256,
        "gasLimit": index_js_1.FieldTypeUint64,
        "to": index_js_1.FieldTypeAddress,
        "value": index_js_1.FieldTypeUint256,
        "from": index_js_1.FieldTypeAddress,
        "data": index_js_1.FieldTypeBytes,
        "feeRatio": index_js_1.FieldTypeUint8,
        "chainId": index_js_1.FieldTypeUint64,
        "txSignatures": index_js_1.FieldTypeSignatureTuples,
        "feePayer": index_js_1.FieldTypeAddress,
        "feePayerSignatures": index_js_1.FieldTypeSignatureTuples,
    };
    // SigRLP = encode([encode([type, nonce, gasPrice, gas, to, value, from, input, feeRatio]), chainid, 0, 0])
    sigRLP() {
        return this.encodeNestedRLP(["type", "nonce", "gasPrice", "gasLimit", "to", "value", "from", "data", "feeRatio"], ["chainId"]);
    }
    // SigFeePayerRLP = encode([encode([type, nonce, gasPrice, gas, to, value, from, input, feeRatio]), feePayer, chainid, 0, 0])
    sigFeePayerRLP() {
        return this.encodeNestedRLP(["type", "nonce", "gasPrice", "gasLimit", "to", "value", "from", "data", "feeRatio"], ["feePayer", "chainId"]);
    }
    // SenderTxHashRLP = type + encode([nonce, gasPrice, gas, to, value, from, input, feeRatio, txSignatures])
    senderTxHashRLP() {
        return this.encodeTypePrefixedRLP(["nonce", "gasPrice", "gasLimit", "to", "value", "from", "data", "feeRatio", "txSignatures"]);
    }
    // TxHashRLP = type + encode([nonce, gasPrice, gas, to, value, from, input, feeRatio, txSignatures, feePayer, feePayerSignatures])
    txHashRLP() {
        return this.encodeTypePrefixedRLP(["nonce", "gasPrice", "gasLimit", "to", "value", "from", "data", "feeRatio", "txSignatures", "feePayer", "feePayerSignatures"]);
    }
    // SenderTxHashRLP = type + encode([nonce, gasPrice, gas, to, value, from, input, feeRatio, txSignatures])
    // TxHashRLP = type + encode([nonce, gasPrice, gas, to, value, from, input, feeRatio, txSignatures, feePayer, feePayerSignatures])
    setFieldsFromRLP(rlp) {
        this.decodeTypePrefixedVarlenRLP(rlp, ["nonce", "gasPrice", "gasLimit", "to", "value", "from", "data", "feeRatio", "txSignatures"], ["nonce", "gasPrice", "gasLimit", "to", "value", "from", "data", "feeRatio", "txSignatures", "feePayer", "feePayerSignatures"]);
    }
}
exports.TxTypeFeeDelegatedSmartContractExecutionWithRatio = TxTypeFeeDelegatedSmartContractExecutionWithRatio;
// https://docs.kaia.io/learn/transactions/partial-fee-delegation#txtypefeedelegatedaccountupdatewithratio-
class TxTypeFeeDelegatedAccountUpdateWithRatio extends abstract_js_1.KlaytnTx {
    static type = index_js_2.TxType.FeeDelegatedAccountUpdateWithRatio;
    static typeName = "TxTypeFeeDelegatedAccountUpdateWithRatio";
    static fieldTypes = {
        "type": index_js_1.FieldTypeUint8,
        "nonce": index_js_1.FieldTypeUint64,
        "gasPrice": index_js_1.FieldTypeUint256,
        "gasLimit": index_js_1.FieldTypeUint64,
        "from": index_js_1.FieldTypeAddress,
        "key": index_js_1.FieldTypeAccountKey,
        "feeRatio": index_js_1.FieldTypeUint8,
        "chainId": index_js_1.FieldTypeUint64,
        "txSignatures": index_js_1.FieldTypeSignatureTuples,
        "feePayer": index_js_1.FieldTypeAddress,
        "feePayerSignatures": index_js_1.FieldTypeSignatureTuples,
    };
    // SigRLP = encode([encode([type, nonce, gasPrice, gas, from, rlpEncodedKey, feeRatio]), chainid, 0, 0])
    sigRLP() {
        return this.encodeNestedRLP(["type", "nonce", "gasPrice", "gasLimit", "from", "key", "feeRatio"], ["chainId"]);
    }
    // SigFeePayerRLP = encode([encode([type, nonce, gasPrice, gas, from, rlpEncodedKey, feeRatio]), feePayer, chainid, 0, 0])
    sigFeePayerRLP() {
        return this.encodeNestedRLP(["type", "nonce", "gasPrice", "gasLimit", "from", "key", "feeRatio"], ["feePayer", "chainId"]);
    }
    // SenderTxHashRLP = type + encode([nonce, gasPrice, gas, from, rlpEncodedKey, feeRatio, txSignatures])
    senderTxHashRLP() {
        return this.encodeTypePrefixedRLP(["nonce", "gasPrice", "gasLimit", "from", "key", "feeRatio", "txSignatures"]);
    }
    // TxHashRLP = type + encode([nonce, gasPrice, gas, from, rlpEncodedKey, feeRatio, txSignatures, feePayer, feePayerSignatures])
    txHashRLP() {
        return this.encodeTypePrefixedRLP(["nonce", "gasPrice", "gasLimit", "from", "key", "feeRatio", "txSignatures", "feePayer", "feePayerSignatures"]);
    }
    // SenderTxHashRLP = type + encode([nonce, gasPrice, gas, from, rlpEncodedKey, feeRatio, txSignatures])
    // TxHashRLP = type + encode([nonce, gasPrice, gas, from, rlpEncodedKey, feeRatio, txSignatures, feePayer, feePayerSignatures])
    setFieldsFromRLP(rlp) {
        this.decodeTypePrefixedVarlenRLP(rlp, ["nonce", "gasPrice", "gasLimit", "from", "key", "feeRatio", "txSignatures"], ["nonce", "gasPrice", "gasLimit", "from", "key", "feeRatio", "txSignatures", "feePayer", "feePayerSignatures"]);
    }
}
exports.TxTypeFeeDelegatedAccountUpdateWithRatio = TxTypeFeeDelegatedAccountUpdateWithRatio;
// https://docs.kaia.io/learn/transactions/partial-fee-delegation#txtypefeedelegatedcancelwithratio-
class TxTypeFeeDelegatedCancelWithRatio extends abstract_js_1.KlaytnTx {
    static type = index_js_2.TxType.FeeDelegatedCancelWithRatio;
    static typeName = "TxTypeFeeDelegatedCancelWithRatio";
    static fieldTypes = {
        "type": index_js_1.FieldTypeUint8,
        "nonce": index_js_1.FieldTypeUint64,
        "gasPrice": index_js_1.FieldTypeUint256,
        "gasLimit": index_js_1.FieldTypeUint64,
        "from": index_js_1.FieldTypeAddress,
        "feeRatio": index_js_1.FieldTypeUint8,
        "chainId": index_js_1.FieldTypeUint64,
        "txSignatures": index_js_1.FieldTypeSignatureTuples,
        "feePayer": index_js_1.FieldTypeAddress,
        "feePayerSignatures": index_js_1.FieldTypeSignatureTuples,
    };
    // SigRLP = encode([encode([type, nonce, gasPrice, gas, from, feeRatio]), chainid, 0, 0])
    sigRLP() {
        return this.encodeNestedRLP(["type", "nonce", "gasPrice", "gasLimit", "from", "feeRatio"], ["chainId"]);
    }
    // SigFeePayerRLP = encode([encode([type, nonce, gasPrice, gas, from, feeRatio]), feePayer, chainid, 0, 0])
    sigFeePayerRLP() {
        return this.encodeNestedRLP(["type", "nonce", "gasPrice", "gasLimit", "from", "feeRatio"], ["feePayer", "chainId"]);
    }
    // SenderTxHashRLP = type + encode([nonce, gasPrice, gas, from, feeRatio, txSignatures])
    senderTxHashRLP() {
        return this.encodeTypePrefixedRLP(["nonce", "gasPrice", "gasLimit", "from", "feeRatio", "txSignatures"]);
    }
    // TxHashRLP = type + encode([nonce, gasPrice, gas, from, feeRatio, txSignatures, feePayer, feePayerSignatures])
    txHashRLP() {
        return this.encodeTypePrefixedRLP(["nonce", "gasPrice", "gasLimit", "from", "feeRatio", "txSignatures", "feePayer", "feePayerSignatures"]);
    }
    // SenderTxHashRLP = type + encode([nonce, gasPrice, gas, from, feeRatio, txSignatures])
    // TxHashRLP = type + encode([nonce, gasPrice, gas, from, feeRatio, txSignatures, feePayer, feePayerSignatures])
    setFieldsFromRLP(rlp) {
        this.decodeTypePrefixedVarlenRLP(rlp, ["nonce", "gasPrice", "gasLimit", "from", "feeRatio", "txSignatures"], ["nonce", "gasPrice", "gasLimit", "from", "feeRatio", "txSignatures", "feePayer", "feePayerSignatures"]);
    }
}
exports.TxTypeFeeDelegatedCancelWithRatio = TxTypeFeeDelegatedCancelWithRatio;
