"use strict";
// type utils
Object.defineProperty(exports, "__esModule", { value: true });
exports.get = exports.has = exports.map = exports.isNumber = exports.isFunction = exports.isArray = exports.isString = void 0;
/**
 * Determines whether the given value is of type string.
 *
 * @param {any} value - The value to check.
 * @returns {boolean} - `true` if the value is a string, otherwise `false`.
 */
const isString = (value) => typeof value === 'string';
exports.isString = isString;
/**
 * Determines whether the given value is an array.
 *
 * @param value - The value to check.
 * @returns {boolean} - `true` if the value is an array, otherwise `false`.
 */
const isArray = (value) => Array.isArray(value);
exports.isArray = isArray;
/**
 * Determines whether the provided value is a function.
 *
 * @param value - The value to check.
 * @returns {boolean} - `true` if the value is a function, otherwise `false`.
 */
const isFunction = (value) => typeof value === 'function';
exports.isFunction = isFunction;
/**
 * Determines if the provided value is of type number.
 *
 * @param {any} value - The value to check.
 * @returns {boolean} - `true` if the value is a number, otherwise `false`.
 */
const isNumber = (value) => typeof value === 'number';
exports.isNumber = isNumber;
// array utils
/**
 * Applies a callback function to each element of an array and returns a new array.
 *
 * @template T - Type of array elements.
 * @param {T[]} array - Input array.
 * @param {(value: T, index: number) => any} callback - Function applied to each element.
 * @returns {any[]} - A new array of transformed elements.
 */
const map = (array, callback) => array.map(callback);
exports.map = map;
/**
 * Checks if the specified key exists as a property in the given object and is not `undefined`.
 *
 * @param {any} obj - The object to inspect.
 * @param {string} key - The property name to check for existence in the object.
 * @returns {boolean} Returns `true` if the key exists and is not `undefined`, otherwise `false`.
 */
const has = (obj, key) => typeof obj === 'object' && typeof obj[key] !== 'undefined';
exports.has = has;
/**
 * Retrieves the value associated with the specified key in an object.
 * If the key does not exist or the input is not an object, returns undefined.
 *
 * @param {any} obj - The object from which to retrieve the value.
 * @param {string} key - The key for which the corresponding value is to be fetched.
 * @returns {any} The value associated with the key if it exists, otherwise undefined.
 */
const get = (obj, key) => typeof obj === 'object' && typeof obj[key] !== 'undefined' ? obj[key] : undefined;
exports.get = get;
