"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.decodeObjectFromRLP = exports.GetHexlifyRLP = exports.convertKeysToRLP = exports.getWeightedPublicKeyRLP = void 0;
const const_js_1 = require("./const.js");
const ec_js_1 = require("./ec.js");
const data_js_1 = require("./data.js");
const helpers_js_1 = require("./helpers.js");
const getWeightedPublicKeyRLP = (tupleOrObject) => {
    if ((0, helpers_js_1.isArray)(tupleOrObject) && tupleOrObject.length == 2) {
        const tuple = tupleOrObject;
        return [
            data_js_1.HexStr.fromNumber(tuple[0]),
            (0, ec_js_1.getCompressedPublicKey)(tuple[1])
        ];
    }
    else if ((0, helpers_js_1.has)(tupleOrObject, "weight") && (0, helpers_js_1.has)(tupleOrObject, "key")) {
        const object = tupleOrObject;
        return [
            data_js_1.HexStr.fromNumber(object.weight),
            (0, ec_js_1.getCompressedPublicKey)(object.key)
        ];
    }
    else {
        throw new Error("Malformed WeightedPublicKeys");
    }
};
exports.getWeightedPublicKeyRLP = getWeightedPublicKeyRLP;
const convertKeysToRLP = ({ key, keys, type, threshold }) => {
    if ((0, helpers_js_1.isArray)(keys)) {
        switch (Number(type)) {
            case const_js_1.AccountKeyType.WeightedMultiSig:
                return data_js_1.RLP.encode([data_js_1.HexStr.from(threshold), keys.map((tupleOrObject) => (0, exports.getWeightedPublicKeyRLP)(tupleOrObject))]);
            case const_js_1.AccountKeyType.RoleBased:
                return data_js_1.RLP.encode(keys.map(({ key, keys, type, threshold }) => (0, exports.GetHexlifyRLP)(type, (0, exports.convertKeysToRLP)({ type, key, keys, threshold }))));
        }
    }
    else {
        switch (Number(type)) {
            case const_js_1.AccountKeyType.Fail:
                return "0x03c0";
            case const_js_1.AccountKeyType.Legacy:
                return "0x01c0";
            case const_js_1.AccountKeyType.Nil:
                return "0x80";
            case const_js_1.AccountKeyType.Public:
                return data_js_1.RLP.encode((0, ec_js_1.getCompressedPublicKey)(key));
        }
    }
    throw new Error("Invalid account key type");
};
exports.convertKeysToRLP = convertKeysToRLP;
const GetHexlifyRLP = (type, rawRLP) => {
    switch (type) {
        case const_js_1.AccountKeyType.RoleBased:
            return data_js_1.HexStr.concat(data_js_1.HexStr.from(const_js_1.AccountKeyType.RoleBased), rawRLP);
        case const_js_1.AccountKeyType.WeightedMultiSig:
            return data_js_1.HexStr.concat(data_js_1.HexStr.from(const_js_1.AccountKeyType.WeightedMultiSig), rawRLP);
        case const_js_1.AccountKeyType.Public:
            return data_js_1.HexStr.concat(data_js_1.HexStr.from(const_js_1.AccountKeyType.Public), rawRLP);
    }
    return rawRLP;
};
exports.GetHexlifyRLP = GetHexlifyRLP;
const decodeObjectFromRLP = (rlp) => {
    const type = data_js_1.HexStr.toNumber(rlp.substring(0, 4));
    switch (type) {
        case const_js_1.AccountKeyType.RoleBased:
            {
                const roleBasedInnerKeys = data_js_1.RLP.decode('0x' + rlp.substring(4));
                return {
                    type: const_js_1.AccountKeyType.RoleBased,
                    keys: roleBasedInnerKeys.map((key) => (0, exports.decodeObjectFromRLP)(key))
                };
            }
        case const_js_1.AccountKeyType.WeightedMultiSig:
            {
                const [threshold, keys] = data_js_1.RLP.decode('0x' + rlp.substring(4));
                return {
                    type: const_js_1.AccountKeyType.WeightedMultiSig,
                    threshold: Number(threshold),
                    keys: keys.map(([weight, innerKey]) => ({ weight: Number(weight), key: innerKey }))
                };
            }
        case const_js_1.AccountKeyType.Public:
            return {
                type: const_js_1.AccountKeyType.Public,
                key: data_js_1.RLP.decode("0x" + String(rlp).substring(4))
            };
        case const_js_1.AccountKeyType.Nil:
            return {
                type: const_js_1.AccountKeyType.Nil,
                key: "0x80"
            };
        case const_js_1.AccountKeyType.Legacy:
            return {
                type: const_js_1.AccountKeyType.Legacy,
                key: "0x03c0"
            };
        case const_js_1.AccountKeyType.Fail:
            return {
                type: const_js_1.AccountKeyType.Fail,
                key: "0x03c0"
            };
    }
    throw new Error("Parse Account Key from RLP errors: Invalid Account key type!");
};
exports.decodeObjectFromRLP = decodeObjectFromRLP;
