"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getRpcTxObject = getRpcTxObject;
const bignumber_1 = require("@ethersproject/bignumber");
const bytes_1 = require("@ethersproject/bytes");
const ethers_1 = require("ethers");
const helpers_js_1 = require("./helpers.js");
const numericFields = ["chainId", "gasLimit", "gasPrice", "type", "maxFeePerGas", "maxPriorityFeePerGas", "nonce", "value"];
const bytestrFields = ["from", "to", "data", "input"];
// Normalize Tx object to suitable for JSON-RPCs such as eth_call and eth_estimateGas.
function getRpcTxObject(tx) {
    const formatted = {};
    let value;
    numericFields.forEach((key) => {
        if (!(0, helpers_js_1.has)(tx, key)) {
            return;
        }
        if (tx[key] == "0x") {
            value = (0, bytes_1.hexValue)("0x");
        }
        else {
            value = (0, bytes_1.hexValue)(bignumber_1.BigNumber.from(tx[key]));
        }
        if (key == "gasLimit") {
            formatted["gas"] = value;
        }
        else {
            formatted[key] = value;
        }
    });
    bytestrFields.forEach((key) => {
        if (!(0, helpers_js_1.has)(tx, key)) {
            return;
        }
        formatted[key] = (0, bytes_1.hexlify)(tx[key]);
        if ((key === "to" || key === "from") && tx[key] === "0x") {
            formatted[key] = "0x0000000000000000000000000000000000000000";
        }
    });
    if (tx.accessList) {
        formatted["accessList"] = (0, ethers_1.accessListify)(tx.accessList);
    }
    return formatted;
}
