"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.clone = exports.toUpperFirst = exports.toCamelCase = exports.toSnakeCase = void 0;
const lodash_snakecase_1 = __importDefault(require("lodash.snakecase"));
const lodash_camelcase_1 = __importDefault(require("lodash.camelcase"));
/**
 * Converts a given camelCase or PascalCase string to snake_case.
 *
 * This function identifies uppercase letters in the input string
 * and replaces them with their lowercase equivalent, prefixed by an underscore.
 * It is commonly used to convert JavaScript variable names to snake_case,
 * which is a standard format in languages like Python or for database field names.
 *
 * @param {string} str - The input string to be converted.
 * @returns {string} The string converted to snake_case.
 */
const toSnakeCase = (str) => (0, lodash_snakecase_1.default)(str);
exports.toSnakeCase = toSnakeCase;
/**
 * Converts a given string with underscores into camelCase format.
 *
 * This function processes an input string, identifies substrings separated by underscores,
 * and capitalizes the first letter of each substring following an underscore. The underscores
 * are removed in the resulting string.
 *
 * @param {string} str - The input string containing words separated by underscores.
 * @returns {string} - The converted string in camelCase format.
 */
const toCamelCase = (str) => (0, lodash_camelcase_1.default)(str);
exports.toCamelCase = toCamelCase;
const toUpperFirst = (str) => str.charAt(0).toUpperCase() + str.slice(1);
exports.toUpperFirst = toUpperFirst;
/**
 * Creates a deep copy of the provided object.
 *
 * @param {object} obj - The object to be cloned.
 * @returns {object} - A new object that is a deep copy of the input object.
 */
const clone = (obj) => {
    return JSON.parse(JSON.stringify(obj));
};
exports.clone = clone;
