/**
 * Determines whether the given value is of type string.
 *
 * @param {any} value - The value to check.
 * @returns {boolean} - `true` if the value is a string, otherwise `false`.
 */
export declare const isString: (value: any) => value is string;
/**
 * Determines whether the given value is an array.
 *
 * @param value - The value to check.
 * @returns {boolean} - `true` if the value is an array, otherwise `false`.
 */
export declare const isArray: (value: any) => value is any[];
/**
 * Determines whether the provided value is a function.
 *
 * @param value - The value to check.
 * @returns {boolean} - `true` if the value is a function, otherwise `false`.
 */
export declare const isFunction: (value: any) => value is CallableFunction;
/**
 * Determines if the provided value is of type number.
 *
 * @param {any} value - The value to check.
 * @returns {boolean} - `true` if the value is a number, otherwise `false`.
 */
export declare const isNumber: (value: any) => value is number;
/**
 * Applies a callback function to each element of an array and returns a new array.
 *
 * @template T - Type of array elements.
 * @param {T[]} array - Input array.
 * @param {(value: T, index: number) => any} callback - Function applied to each element.
 * @returns {any[]} - A new array of transformed elements.
 */
export declare const map: <T>(array: T[], callback: (value: T, index: number) => any) => any[];
/**
 * Checks if the specified key exists as a property in the given object and is not `undefined`.
 *
 * @param {any} obj - The object to inspect.
 * @param {string} key - The property name to check for existence in the object.
 * @returns {boolean} Returns `true` if the key exists and is not `undefined`, otherwise `false`.
 */
export declare const has: (obj: any, key: string) => boolean;
/**
 * Retrieves the value associated with the specified key in an object.
 * If the key does not exist or the input is not an object, returns undefined.
 *
 * @param {any} obj - The object from which to retrieve the value.
 * @param {string} key - The key for which the corresponding value is to be fetched.
 * @returns {any} The value associated with the key if it exists, otherwise undefined.
 */
export declare const get: (obj: any, key: string) => any;
